@@ -10,18 +10,60 @@ import operator
 import plotly.graph_objects as go
 
 # Mapper from matplotlib to plotly
-line_style_mapper = {
+COLOR_MAPPER = {
+    'b': 'rgb(0, 0, 255)',
+    'blue': 'rgb(0, 0, 255)',
+    'g': 'rgb(0, 128, 0)',
+    'green': 'rgb(0, 128, 0)',
+    'r': 'rgb(255, 0, 0)',
+    'red': 'rgb(255, 0, 0)',
+    'c': 'rgb(0, 255, 255)',
+    'cyan': 'rgb(0, 255, 255)',
+    'm': 'rgb(255, 0, 255)',
+    'magenta': 'rgb(255, 0, 255)',
+    'y': 'rgb(255, 255, 0)',
+    'yellow': 'rgb(255, 255, 0)',
+    'k': 'rgb(0, 0, 0)',
+    'black': 'rgb(0, 0, 0)',
+    'w': 'rgb(255, 255, 255)',
+    'white': 'rgb(255, 255, 255)',
+}
+
+LINE_STYLE_MAPPER = {
     '': 'solid',
     '--': 'dash',
     ':': 'dot',
     '-.': 'dashdot',
 }
 
-marker_style_mapper = {
+MARKER_STYLE_MAPPER = {
+    '.': 'circle',
+    ',': 'circle',
     'o': 'circle',
-    '^': 'triangle-up',
     'v': 'triangle-down',
+    '^': 'triangle-up',
+    '<': 'triangle-left',
+    '>': 'triangle-right',
+    '1': 'y-down',
+    '2': 'y-up',
+    '3': 'y-left',
+    '4': 'y-right',
+    '8': 'octagon',
+    's': 'square',
+    'p': 'pentagon',
+    'P': 'cross',
+    '*': 'star',
+    'h': 'hexagon',
+    'H': 'hexagon2',
+    '+': 'cross-thin',
+    'x': 'x-thin',
+    'X': 'x',
+    'D': 'diamond',
+    'd': 'diamond-tall',
+    '|': 'line-ns',
+    '_': 'line-ew',
 
+    # custom markers mapping
     '$\u21E7$': 'triangle-up',
     '$\u21E9$': 'triangle-down',
 }
@@ -81,6 +123,7 @@ class BacktraderPlotly(metaclass=bt.MetaParams):
     )
 
     def __init__(self, **kwargs):
+        self.figs = []
         for pname, pvalue in kwargs.items():
             setattr(self.p.scheme, pname, pvalue)
 
@@ -186,6 +229,8 @@ class BacktraderPlotly(metaclass=bt.MetaParams):
             # for i in range(self.pinf.nrows):
             #     self.fig['layout'][f'xaxis{i + 1}']['showticklabels'] = True
 
+        self.figs.extend(figs)
+
         return figs
 
     def newaxis(self, obj, rowspan):
@@ -254,12 +299,9 @@ class BacktraderPlotly(metaclass=bt.MetaParams):
             if lineplotinfo._get('_plotskip', False):
                 continue
 
-            # Legend label only when plotting 1st line
-            if masterax and not ind.plotinfo.plotlinelabels:
-                label = indlabel * (not toskip) or '_nolegend'
-            else:
-                label = (indlabel + '\n') * (not toskip)
-                label += lineplotinfo._get('_name', '') or linealias
+            # Always give legend label
+            label = (indlabel + '\n') * (not toskip)
+            label += lineplotinfo._get('_name', '') or linealias
 
             toskip -= 1  # one line less until legend can be added
 
@@ -325,7 +367,7 @@ class BacktraderPlotly(metaclass=bt.MetaParams):
                         l2 = getattr(ind, fref)
                         prl2 = l2.plotrange(self.pinf.xstart, self.pinf.xend)
                         y2 = np.array(prl2)
-                    kwargs = dict()
+                    kwargs = dict(plotkwargs)
                     if fop is not None:
                         kwargs['where'] = fop(y1, y2)
 
@@ -333,11 +375,14 @@ class BacktraderPlotly(metaclass=bt.MetaParams):
                     if isinstance(fcol, (list, tuple)):
                         fcol, falpha = fcol
 
-                    ax.fill_between(self.pinf.xdata, y1, y2,
-                                    facecolor=fcol,
-                                    alpha=falpha,
-                                    interpolate=True,
-                                    **kwargs)
+                    kwargs['color'] = fcol
+                    kwargs['opacity'] = falpha
+                    kwargs['attr'] = fattr
+
+                    if fop is operator.lt:
+                        y1, y2 = y2, y1  # always set such that y1 >= y2 for correct plotting
+
+                    self.fill_between(ax, xdata, y1, y2, secondary_y=secondary_y, **kwargs)
 
         # plot subindicators that were created on self
         for subind in subinds:
@@ -351,10 +396,11 @@ class BacktraderPlotly(metaclass=bt.MetaParams):
         # print(ax, plotkwargs)
 
         opacity = 1
-        line = dict(color=plotkwargs['color'])  # line or marker style
+        color = self.convert_color_syntax(plotkwargs['color'])
+        line = dict(color=color, width=2)
         if 'marker' in plotkwargs:
             # Scatter plot
-            marker = dict(symbol=marker_style_mapper.get(plotkwargs['marker'], 0))
+            marker = dict(symbol=MARKER_STYLE_MAPPER.get(plotkwargs['marker'], 0), line=line)
             self.fig.add_trace(go.Scatter(mode='markers',
                                           x=np.array(xdata),
                                           y=np.array(lplotarray),
@@ -372,13 +418,13 @@ class BacktraderPlotly(metaclass=bt.MetaParams):
                                       y=np.array(lplotarray),
                                       name=self.wrap_legend_text(plotkwargs['label']),
                                       opacity=opacity,
-                                      width=plotkwargs['width'],
+                                      marker_color=color,
                                       ), row=ax, col=1, secondary_y=secondary_y
                                )
         else:
             # Line plot
             if 'ls' in plotkwargs:
-                line['dash'] = line_style_mapper[plotkwargs['ls']]
+                line['dash'] = LINE_STYLE_MAPPER[plotkwargs['ls']]
 
             self.fig.add_trace(go.Scatter(x=np.array(xdata),
                                           y=np.array(lplotarray),
@@ -542,7 +588,8 @@ class BacktraderPlotly(metaclass=bt.MetaParams):
 
     def show(self):
         if self.p.show:
-            self.fig.show()
+            for fig in self.figs:
+                fig.show()
 
     def sortdataindicators(self, strategy):
         # These lists/dictionaries hold the subplots that go above each data
@@ -653,3 +700,51 @@ class BacktraderPlotly(metaclass=bt.MetaParams):
             return '<br>'.join(s[i: i + max_length] for i in range(0, n, max_length))
         else:
             return s
+
+    def rgb_to_rgba(self, rgb, opacity):
+        return f'rgba{rgb[3:-1]}, {opacity})'
+
+    def convert_color_syntax(self, color):
+        if color.startswith('C'):
+            # Color is the i-th color in the cycle
+            return COLOR_MAPPER.values()[int(color[1:]) % len(COLOR_MAPPER)]
+
+        elif color in COLOR_MAPPER:
+            # Color is basic built-in colors of matplotlib in one letter
+            return COLOR_MAPPER[color]
+
+        return color
+
+    def fill_between(self, ax, x, y1, y2, secondary_y=False, **kwargs):
+        color = COLOR_MAPPER[kwargs.get('color', 'black')]
+        opacity = kwargs.get('opacity', 1)
+        color = self.rgb_to_rgba(color, opacity)
+
+        x = np.array(x)
+        y1 = np.array(y1)
+        y2 = np.array(y2)
+        where = kwargs.get('where', None)
+        if where is not None:
+            y2[where == False] = y1[where == False]
+
+        name = f'{kwargs.get("attr", "")}_{kwargs.get("label", "")}'
+        legendgroup = f'{name}_{ax}'
+
+        self.fig.add_trace(go.Scatter(x=x,
+                                      y=y2,
+                                      name=self.wrap_legend_text(name),
+                                      legendgroup=legendgroup,
+                                      showlegend=False,
+                                      line=dict(color=color),
+                                      ), row=ax, col=1, secondary_y=secondary_y
+                           )
+
+        self.fig.add_trace(go.Scatter(x=x,
+                                      y=y1,
+                                      name=self.wrap_legend_text(name),
+                                      legendgroup=legendgroup,
+                                      fill='tonexty',
+                                      fillcolor=color,
+                                      line=dict(color=color),
+                                      ), row=ax, col=1, secondary_y=secondary_y
+                           )
